#include <VRS51L3074_SDCC.h>
#include "RTC.h"
#include "i2c.h"


 void Init_RTC (void){


        //--If Using a Capacitor RTC Backup,
	//Activate the FM31xx tricle charger (VBC bit of 0x0B register = 1 ) 
	//RTCWrite(0xD0,0x0B,0x04);
	
	//--If Using a Lithium Battery RTC Backup,
	//De activate the FM31xx triclke charges (VBC bit of 0x0B register = 0 ) 
	//RTCWrite(0xD0,0x0B,0x00);
		

}

//----------------------------------------------------------------------------//
// Function RTCRead(char idw,char address) //
//----------------------------------------//
//	Descrition:
//   
//	Read the register pointed by the variable address of the FM31xx pointed by 
//	by "idw"
//		
//-----------------------------------------------------------------------------//
unsigned char RTCRead(unsigned char idw,unsigned char address){
	I2CRXTX = idw;							//Write I2C device ID + W
	
	WaitTXEMP();
	I2CRXTX = address;						//Write I2C ADRSL

	//--Wait for I2C IDLE  (This will generate a STOP)
	WaitI2CIDLE();


	//--Start a Preset ADRS read (This will generate a START)
	I2CRXTX = idw+1;						//Write I2C device ID + R
	WaitTXEMP();
	I2CCONFIG |= 0x02;						//Force I2C to Not Acknowledge after receiving the next data byte
	WaitRXAV();							//Wait for RX Availlable bit, This will trigger I2C Reception
        I2CCONFIG &= ~0x02;						// Regular Acknowledge 
	return I2CRXTX;						//Return Data Byte

	}//End of RTCRead


//----------------------------------------------------------------------------//
// Function RTCWrite(char idw,char address,char value) //
//--------------------------------------//
//	Descrition:
//   
//	Write value parameter "value" at address "address" into the FM31xx pointed
//	by "idw"
//		
//-----------------------------------------------------------------------------//
void RTCWrite(unsigned char idw,unsigned char address,unsigned char value){
	I2CRXTX = idw;							//Write I2C device ID + W
	
	WaitTXEMP();
	I2CRXTX = address;						//Write I2C adrs
	WaitTXEMP();
	I2CRXTX = value;						//Write I2C data

	//--Wait for I2C IDLE  (This will generate a STOP)
	WaitI2CIDLE();

	}//End of RTCWrite

//-----------------------------------------------------------//
// RTCReadAll(char idw)   //
//------------------------//
// Description:  
//	
//	Read all RTC parameters at once and store them 
// 	into a global char table named "rtcvalue'
//	
//	rtcvalue is defined as follow:
//		rtcvalue[0] = seconds
//		rtcvalue[1] = minutes
//		rtcvalue[2] = hours
//		rtcvalue[3] = day
//		rtcvalue[4] = date
//		rtcvalue[5] = month
//		rtcvalue[6] = Years
//
//	Date:  June, 2006
//--------------------------------------------------------------//
void RTCReadAll(unsigned char idw, unsigned char *rtcvalue){
  
  register unsigned char loop=6;
	
	I2CRXTX = idw;							//Write I2C device ID + W
	WaitTXEMP();
	I2CRXTX = 0x00;						//Write I2C ADRSL = RTC Control
	
	WaitTXEMP();	
	//--Set the RTC R bit to read the parameters
	I2CRXTX = 0x01;						//RTC Control = 0x01 (R bit = 1)
		
	//--Wait for I2C IDLE  (This will generate a STOP)
	WaitI2CIDLE();

	//--Start a Preset ADRS read (This will generate a START)
	I2CRXTX = (idw+1);						//Write I2C device ID + R
	WaitTXEMP();		
	WaitRXAV();							//Dummy read of CAL/CTRL	
	*rtcvalue = I2CRXTX;
	
        while(loop--){
	   WaitRXAV();							//Wait for RX Availlable bit, This will trigger I2C Reception
	   *(rtcvalue++) = I2CRXTX;					//Retrieve the values
	}

	I2CCONFIG |= 0x02;						//Force I2C to Not Acknowledge after receiving the next data byte
	
	WaitRXAV();							//Wait for RX Availlable bit, This will trigger I2C Reception
	*(rtcvalue++) = I2CRXTX;					//Retrieve last value
        I2CCONFIG &= ~0x02;						// Regular Acknowledge 
	
	//--Clear the RTC R bit
	
	I2CRXTX = idw;							//Write I2C device ID + W
	
	WaitTXEMP();
	I2CRXTX = 0x00;						//Write I2C ADRSL = RTC Control
	
	WaitTXEMP();	
	//--Clear the RTC R bit 
	I2CRXTX = 0x00;						//RTC Control = 0x00 (R bit = 0)
		
	//--Wait for I2C IDLE  (This will generate a STOP)
	WaitI2CIDLE();
	
	}//End of RTCReadAll	
	


//----------------------------------------------------------------------------//
// Function RTCSet(char idw,char * rtcparam) //
//--------------------------------------//
//	Descrition:
//   
//	Write current content of RTC parameters table in the FM31xx RTC
//	Input parameter:
//		char idw:	address of FM31xx device
//		char * pointer to the table containing the RTC parameters
//		rtcparam[0] = seconds
//		rtcparam[1] = minutes
//		rtcparam[2] = hours
//		rtcparam[3] = day
//		rtcparam[4] = date
//		rtcparam[5] = month
//		rtcparam[6] = Years
//-----------------------------------------------------------------------------//

void RTCSet(unsigned char idw, unsigned char *rtcparam)
{
  register unsigned char loop=7;
 
	I2CRXTX = idw;							//Write I2C device ID + W
	
	WaitTXEMP();
	I2CRXTX = 0x00;						//address = RTC Control
	WaitTXEMP();
	I2CRXTX = 0x02;						//Set the W bit to update time
	
	WaitTXEMP();
	I2CRXTX	= 0x00;						//Clear the OSCEN bit = Start 32KHz Osc

	while(loop--){
	   WaitTXEMP();							//Wait for TX Empty bit
	   I2CRXTX = *(rtcparam++);					//Store the values
	}
	
	//--Wait for I2C IDLE  (This will generate a STOP)
	WaitI2CIDLE();
	
	//Clear the W bit
	
	I2CRXTX = idw;							//Write I2C device ID + W
	
	WaitTXEMP();
	I2CRXTX = 0x00;						//address = RTC Control
	WaitTXEMP();
	I2CRXTX = 0x00;						//Clear the W bit to update time
	//--Wait for I2C IDLE  (This will generate a STOP)
	WaitI2CIDLE();
	}//end of RTC Set

